// ping.c written by Cronos 08/01/00

#include <stdio.h>
#include <winsock.h>

#define DEFAULT_TTL 100

struct o
{	unsigned char Ttl,Tos,Flags,OptionsSize,*OptionsData;
};

struct
{	DWORD Address;
	unsigned long  Status,RoundTripTime;
	unsigned short DataSize,Reserved;
	void *Data;
	struct o  Options;
} E;

WSADATA wsaData;
HANDLE hIcmp;
struct hostent *s;

HANDLE (WINAPI *pIcmpCreateFile)(VOID);
BOOL (WINAPI *pIcmpCloseHandle)(HANDLE);
DWORD (WINAPI *pIcmpSendEcho)(HANDLE,DWORD,LPVOID,WORD,LPVOID,LPVOID,DWORD,DWORD);

void init_icmp(void)
{  hIcmp=LoadLibrary("ICMP.DLL");
// suppress ptr conversion warnings
#pragma warn -sus
	pIcmpCreateFile=GetProcAddress(hIcmp,"IcmpCreateFile");
	pIcmpCloseHandle=GetProcAddress(hIcmp,"IcmpCloseHandle");
	pIcmpSendEcho=GetProcAddress(hIcmp,"IcmpSendEcho");
#pragma warn +sus
}

void exit_icmp(void)
{  FreeLibrary( hIcmp );
}

int init_winsock(WORD version,BOOL verbose)
{  int err;
	if(verbose) printf("Initialising Winsock\n");
	err = WSAStartup(version, &wsaData);
	if(err)
	{	printf("Could not initialise Winsock\n");
		if(verbose) printf("Error code:%d\n",WSAGetLastError());
	}
	return err;
}

void exit_winsock(void)
{	WSACleanup();
}

unsigned long hostlookupaddr(char *target,BOOL verbose)
{	unsigned long iadr;
	s=NULL;
	iadr=inet_addr(target);
	if(iadr==INADDR_NONE)
	{	if(verbose) printf("Trying gethostbyname\n");
		s=gethostbyname(target);
		if(s==NULL)
		{	printf("No entry for IP...(Error Code:%d)\n",WSAGetLastError());
			return 0;
		}
		return ((unsigned long *)(s->h_addr_list[0]))[0];
	}
	return iadr;
}

void ping_ip(unsigned long dwIPAddr,BOOL verbose,unsigned char ttl,unsigned int npings)
{	HANDLE hIP;
	struct o I;
	unsigned long d;

	if(verbose) printf("Sending ping(s) to :%s\n",inet_ntoa(((struct in_addr *)&dwIPAddr)[0]));
	hIP=pIcmpCreateFile();
	while(npings)
	{  if(verbose) printf("Ping %3d: ",npings);
		I.Ttl=ttl;
		pIcmpSendEcho(hIP,dwIPAddr,0,0,&I,&E,sizeof(E),8000);
		d=E.Address;
      if(!d) printf("Request timed out\n");
      else
		{	s=gethostbyaddr((char *)&d,4,PF_INET);
			if(s==NULL)
				printf("%s reply: ",inet_ntoa(((struct in_addr *)&d)[0]));
			else
				printf("%s reply: ",s->h_name);
			printf("RTT: %dms,TTL:%d\n",E.RoundTripTime,E.Options.Ttl);
      }
		npings--;
	}
	pIcmpCloseHandle(hIP);
}

void helpscreen(void)
{	printf("\nPing by Cronos - help\n");
	printf("---------------------\n");
	printf("Ping sends a ping packet to a target and awaits a reply giving the time of the round trip\n");
	printf("Options:\n");
	printf("-h These help options\n");
	printf("-v Verbose mode (more messages)\n");
	printf("-n<num> Number of pings to send\n");
	printf("-t<num> Time to Live (0-255):Default 100\n");
	printf("Example: ping -v 208.125.234.22\n");
	printf("Example: ping -n5 modem15.dialup.fserve.net\n");
	printf("Example: ping -t20 ftp.acomp.hu\n");
}

unsigned int parsenum(char *sptr)
{  unsigned int pnum;
	pnum=0;
	while((*sptr>='0')&&(*sptr<='9'))
	{	pnum=(unsigned int)(pnum*10+((*sptr)-'0'));
		sptr++;
	}
	return pnum;
}

unsigned char parsecnum(char *sptr)
{  unsigned char pnum;
	pnum=0;
	while((*sptr>='0')&&(*sptr<='9'))
	{	pnum=(unsigned char)(pnum*10+((*sptr)-'0'));
		sptr++;
	}
	return pnum;
}

int main(int argc,char *argv[])
{  char *errv,*arv,*opts;
	int argn,tnum,oc;
	unsigned char ttl;
	unsigned int npings;
	BOOL verb,hp;
	unsigned long padr;

	npings=1;
	ttl=DEFAULT_TTL;
	if (argc < 2)
	{	errv=argv[0];
		arv=errv;
		while(*errv)
		{	if(*errv=='\\') arv=errv+1;
			errv++;
		}
		printf("Usage: %s -[hvn<num>t<num>] <target>\n",arv);
		exit(-1);
	}
	verb=FALSE;
	hp=FALSE;
	argn=1;
	tnum=0;
	while(argn<argc)
	{	opts=argv[argn];
		if((opts[0]!='-')&&(!tnum)) tnum=argn;
		else if(opts[0]=='-')
		{	// parse options.
			oc=1;
			while(opts[oc])
			{  switch(opts[oc])
				{	case 'h':
					case 'H':
						hp=TRUE;
						break;
					case 'v':
					case 'V':
						verb=TRUE;
						break;
					case 't':
					case 'T':
						ttl=parsecnum(&opts[oc+1]);
						while((opts[oc+1]>='0')&&(opts[oc+1]<='9')) oc++;
						break;
					case 'n':
					case 'N':
						npings=parsenum(&opts[oc+1]);
						while((opts[oc+1]>='0')&&(opts[oc+1]<='9')) oc++;
						break;
					default:
						printf("Unknown option:%c\nUse -h for help\n",opts[oc]);
						exit(-1);
				}
				oc++;
			}
		}
		else
		{	printf("Target set as:%s\n",argv[tnum]);
			printf("Found %s in command line as well\n",argv[argn]);
			printf("Use -h for help\n");
			exit(-1);
		}
		argn++;
	}

	if(hp)
	{	helpscreen();
		if(!tnum) exit(0);
	}

	if(!tnum)
	{	printf("No target found in command line\nUse -h for help\n");
		exit(-1);
	}

	if(verb)
	{	printf("Ping by Cronos\n");
		printf("Target         :%s\n",argv[tnum]);
		printf("TTL            :%u\n",ttl);
		printf("Number of Pings:%u\n",npings);
	}

	if(!ttl)
	{	printf("Time to live is zero, exitting\n");
		exit(-1);
	}

	if(!npings)
	{	printf("Number of pings is zero, exitting\n");
		exit(-1);
	}

	if(init_winsock(MAKEWORD(1, 1),verb)) exit(-1);

	padr=hostlookupaddr(argv[tnum],verb);
	if(!padr)
	{	exit_winsock();
		exit(-1);
	}

	init_icmp();
	ping_ip(padr,verb,ttl,npings);
	exit_icmp();

	exit_winsock();
	return 0;
}


